<?php

/**
 * Blesta License Module
 *
 * @copyright Copyright (c) 2019, Phillips Data, Inc.
 * @link http://www.blesta.com/ Blesta
 */
class Blesta extends HostingModule
{
    /**
     * Module version to display in admin area
     *
     * @var string
     */
    protected $version = '1.0';

    /**
     * Module name to display in admin area
     *
     * @var string
     */
    protected $modname = 'Blesta License Module';

    /**
     * Module description to display in admin area
     *
     * @var string
     */
    protected $description = 'Handles the creation, cancellation, suspension, unsuspension, and re-issue of Blesta Licenses.';

    /**
     * Options for the product configuration
     *
     * @var array
     */
    protected $options = [];

    /**
     * Choose which fields to display app settings.
     *
     * @var array
     */
    protected $serverFields = [
        'hostname'    => false,
        'ip'          => false,
        'maxaccounts' => false,
        'status_url'  => false,
        'field1'      => true,
        'username'    => true,
        'password'    => true,
        'hash'        => false,
        'ssl'         => true,
        'nameservers' => false
    ];

    /**
     * HostBill will replace default labels for server fields
     * with this variable configured
     *
     * @var array
     */
    protected $serverFieldsDescription = [
        'username' => 'Username',
        'password' => 'Password',
        'field1'   => 'License ID <br/> <small>(Usually 370 for Monthly Licenses and 364 for VAR Licenses)</small>',
        'ssl'      => 'Test Mode'
    ];

    /**
     * Use $details array to store information returned in Create() method.
     * Use details stored here to identify account for suspension/termination.
     * Add more fields if needed.
     *
     * @see Create()
     * @var array
     */
    protected $details = [
        'option1' => [
            'name'    => 'username',
            'value'   => false,
            'type'    => 'hidden',
            'default' => false
        ],
        'option2' => [
            'name'    => 'password',
            'value'   => false,
            'type'    => 'hidden',
            'default' => false
        ],
        'option3' => [
            'name'    => 'rootpassword',
            'value'   => false,
            'type'    => 'hidden',
            'default' => false
        ],
        'option4' => [
            'name'    => 'domain',
            'value'   => false,
            'type'    => 'hidden',
            'default' => false
        ],
        'option5' => [
            'name'    => 'license',
            'value'   => false,
            'type'    => 'hidden',
            'default' => false
        ]
    ];

    /**
     * The Blesta account username
     *
     * @see connect()
     * @ignore
     * @var string
     */
    private $blesta_username;

    /**
     * The Blesta account password
     *
     * @see connect()
     * @ignore
     * @var string
     */
    private $blesta_password;

    /**
     * License ID
     *
     * @see connect()
     * @ignore
     * @var string
     */
    private $license_id;

    /**
     * Test mode
     *
     * @see connect()
     * @ignore
     * @var string
     */
    private $test_mode;

    /**
     * This method will be called by HostBill before calling any other function from the module.
     *
     * @param array $app_settings App settings configured in Settings->Apps
     */
    public function connect($app_settings)
    {
        $this->blesta_username = $app_settings['username'];
        $this->blesta_password = $app_settings['password'];
        $this->license_id      = $app_settings['field1'];
        $this->test_mode       = $app_settings['ssl'];
    }

    /**
     * HostBill will call this method when admin clicks on "test Connection" in settings->apps
     * It should test connection to remote app using details provided in connect method
     * Use $this->addError('message'); to provide errors details (if any)
     *
     * @see connect
     * @return boolean true if connection suceeds
     */
    public function testConnection()
    {
        // Test API connection
        $connection = $this->Send('testconnection');

        return isset($connection['message']);
    }

    /**
     * This method is invoked automatically when creating an account.
     *
     * @return boolean true if creation succeeds
     */
    public function Create()
    {
        // Issue new license
        $data    = [
            'vars' => [
                'pricing_id' => $this->license_id
            ]
        ];
        $license = $this->Send('addlicense', $data);

        // Save required and additional fields
        if ($license !== false) {
            // Set username field - required by HostBill
            $this->details['option1']['value'] = $license['response'];

            // Set password field - required by HostBill
            $this->details['option2']['value'] = $license['response'];

            // Set root password field - required by HostBill
            $this->details['option3']['value'] = $license['response'];

            // Set domain field - required by HostBill
            $this->details['option4']['value'] = $license['response'];

            // Set license field
            $this->details['option5']['value'] = $license['response'];

            $this->addInfo('License has been issued.');

            return true;
        }

        return false;
    }

    /**
     * This method is invoked automatically when suspending an account.
     *
     * @return boolean true if suspend succeeds
     */
    public function Suspend()
    {
        // Suspend license
        $data    = [
            'vars' => [
                'license' => $this->details['option5']['value']
            ]
        ];
        $license = $this->Send('suspendlicense', $data);

        return ($license !== false);
    }

    /**
     * This method is invoked automatically when unsuspending an account.
     *
     * @return boolean true if unsuspend succeeds
     */
    public function Unsuspend()
    {
        // Unsuspend license
        $data    = [
            'vars' => [
                'license' => $this->details['option5']['value']
            ]
        ];
        $license = $this->Send('unsuspendlicense', $data);

        return ($license !== false);
    }

    /**
     * This method is invoked automatically when terminating an account.
     *
     * @return boolean true if termination succeeds
     */
    public function Terminate()
    {
        // Terminate license
        $data    = [
            'vars' => [
                'license' => $this->details['option5']['value']
            ]
        ];
        $license = $this->Send('cancellicense', $data);

        return ($license !== false);
    }

    /**
     * This method is invoked when the license should be re-issued
     *
     * @return boolean true if action succeeded
     */
    public function Reissue()
    {
        // Re-issue license
        $data    = [
            'vars' => [
                'license'        => $this->details['option5']['value'],
                'reissue_status' => 'reissue'
            ]
        ];
        $license = $this->Send('update', $data);

        $this->addInfo('License has been re-issued.');

        return ($license !== false);
    }

    /**
     * This method is invoked when the license details are requested
     *
     * @return array an array containing the license details
     */
    public function LicenseDetails()
    {
        $data = [
            'license_key' => $this->details['option5']['value'],
            'license_id'  => $this->license_id,
            'reseller'    => $this->blesta_username
        ];

        return $data;
    }

    /**
     * This method is invoked when account should have password changed
     *
     * @return boolean true if action succeeded
     */
    public function ChangePassword()
    {
        $this->Reissue();

        return false;
    }

    /**
     * Components:Forms upgrade/downgrade.
     * if upgrade logic is in ChangePackage there is no need to edit function below.
     *
     * @param array $account_config New account config, under $this->account_config there is old one
     *
     * @return boolean
     */
    public function changeFormsFields($account_config)
    {
        if (empty($account_config)) {
            return true;
        }

        $this->setAccountConfig(array_merge($this->account_config, $account_config));

        return $this->ChangePackage();
    }

    /**
     * This method is invoked when account should be upgraded/downgraded
     * $options variable is loaded with new package configuration
     *
     * @return boolean true if action succeeded
     */
    public function ChangePackage()
    {
        return true;
    }

    /**
     * Send a request to the Blesta Reseller API
     *
     * @ignore
     */
    public function Send($action, array $vars = [])
    {
        // API URL
        $url = 'https://account.blesta.com/plugin/blesta_reseller/v2/index/' . $action . '.json';

        // Enable test mode
        $vars['vars']['test_mode'] = (bool) $this->test_mode ? 'true' : 'false';

        // Process curl request to perform this action
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($vars));
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_USERPWD, $this->blesta_username . ':' . $this->blesta_password);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

        $response      = curl_exec($ch);
        $response_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        // Decode the response
        $result = json_decode($response, true);

        // Add the HTTP code to the response
        $result['success'] = ($response_code == 200) ? true : false;

        // Log result
        if (!$response_code) {
            $this->addError($result['message']);

            return false;
        }

        return $result;
    }
}